#include "GBoard.h"
#include "SStringList.h"
#include "main.h"
#include "GPackage.h"


GBoard::GBoard()
{
	m_nMaxWeight=0;
}

int FindBufLength(const char* sBuf, int nSize)
{
	int nLen=nSize;
	for(int i=0;i<nSize;i++)
		if(sBuf[i]!='~' && sBuf[i]!='#' && sBuf[i]!='.' && sBuf[i]!='@')
		{
			nLen=i;
			break;
		}

	return nLen;
}

void GBoard::ReceiveBoard(SSocket* pSocket)
{
	char sBuf[1024];
	unsigned int nSize=1024;
	int nRow, nCol;
	pSocket->RecvUntilStr(sBuf, &nSize, "\n", 1, FALSE,  0);
	sBuf[nSize]=NULL;

	SStringList sl(sBuf, ' ');
	m_nWidth=sl.GetItemAsInt(0);
	m_nHeight=sl.GetItemAsInt(1);

	G_Printf("Board Size:%ix%i\n", m_nWidth, m_nHeight);
	m_pBoard=new char*[m_nHeight];
	int nLastRead=0;
	nRow=0;
	//for(nRow=0;nRow<m_nHeight;nRow++)
	while(1)
	{
		m_pBoard[nRow]=new char[m_nWidth+1];
		nSize=1024;
		pSocket->RecvUntilStr(sBuf, &nSize, "\n", 1, FALSE,  0);
		nSize=FindBufLength(sBuf+nLastRead, nSize);
		nLastRead+=nSize;
		//nLastRead+=pSocket->Recv(m_pBoard[nRow]+nLastRead, nSize, 0);
		if(nLastRead==m_nWidth)
		{
			memcpy(m_pBoard[nRow], sBuf, nLastRead);
			m_pBoard[nRow][nLastRead]=NULL;
			nLastRead=0;			

			// find any home bases in this row
			for(nCol=0;nCol<m_nWidth;nCol++)
				if(m_pBoard[nRow][nCol]=='@')
					m_HomeBase.AddObj(m_HomeBase.Count(), new GHomeBase(m_HomeBase.Count(), nCol+1,nRow+1));

			G_Printf("%s\n", m_pBoard[nRow]);
			nRow++;
			if(nRow==m_nHeight)
				break;
		}		
	}
}

int GBoard::HomeBaseX(int nID)
{
	GHomeBase* pBase=(GHomeBase*)m_HomeBase.FindObj(nID);
	return pBase->m_nX;
}

int GBoard::HomeBaseY(int nID)
{
	GHomeBase* pBase=(GHomeBase*)m_HomeBase.FindObj(nID);
	return pBase->m_nY;
}

GHomeBase* GBoard::IsHomeBase(int nX, int nY)
{
	if(m_pBoard[nY-1][nX-1]=='@')
	{
		GHomeBase* pBase=(GHomeBase*) m_HomeBase.FirstObj();
		while(pBase)
		{
			if(pBase->m_nX==nX && pBase->m_nY==nY)
			{		
				return pBase;
			}

			pBase=(GHomeBase*) m_HomeBase.NextObj();		
		}		
	}
	return NULL;
}

void GBoard::ResetBasePackageCount(int nX, int nY)
{
	GHomeBase* pBase=IsHomeBase(nX, nY);
	if(pBase)// && pBase->m_nPackageCnt==-1)
		pBase->m_nPackageCnt=0;
}

void GBoard::AddPackage(int nPackage, int nSX, int nSY, int nDestX, int nDestY, int nWeight)
{
	GHomeBase* pBase=IsHomeBase(nSX, nSY);
	GPackage* pPackage=(GPackage*)m_Packages.FindObj(nPackage);
	if(pBase && pBase->m_nPackageCnt==-1)
		pBase->m_nPackageCnt=0;
	if(!pPackage)
	{
		pPackage=new GPackage(nPackage, this);
		pPackage->Init(nSX, nSY, nDestX, nDestY, nWeight);

		if(pPackage->m_nWeight>0 && pPackage->m_nWeight<=m_nMaxWeight && pPackage->m_nDistance>0)
		{
			m_Packages.AddObj(nPackage, (void*) pPackage);		
			if(pBase)
			{
				pBase->m_nPackageCnt++;
			}
			G_Printf("Added Package: ID=%i at(%i,%i) to(%i,%i) W=%i D=%i\n", nPackage, nSX, nSY, nDestX, nDestY, nWeight, pPackage->m_nDistance);		
		}
		else
		{
			G_Printf("Invalid Package: ID=%i at(%i,%i) to(%i,%i) W=%i D=%i\n", nPackage, nSX, nSY, nDestX, nDestY, nWeight, pPackage->m_nDistance);
			delete pPackage;
		}
	}
	else
	{
		if(pBase)
			pBase->m_nPackageCnt++;
		if(pPackage->m_nX!=nSX || pPackage->m_nY!=nSY || pPackage->m_nDestX!=nDestX || pPackage->m_nDestY!=nDestY)
		{
			pPackage->Init(nSX, nSY, nDestX, nDestY, nWeight);
			G_Printf("Updated Package: ID=%i at(%i,%i) to(%i,%i) W=%i D=%i\n", nPackage, nSX, nSY, nDestX, nDestY, nWeight, pPackage->m_nDistance);
		}
		else
			G_Printf("Known Package: ID=%i at(%i,%i) to(%i,%i) W=%i D=%i\n", nPackage, nSX, nSY, nDestX, nDestY, nWeight, pPackage->m_nDistance);
	}
	
}

void GBoard::ResetPickedPackages()
{	
	
	GPackage* pPackage=(GPackage*) m_Packages.FirstObj();
	while(pPackage)
	{
		pPackage->m_bPicked=FALSE;
		pPackage=(GPackage*) m_Packages.NextObj();		
	}
}

GPackage* GBoard::FindBestDelivery(int nSX, int nSY, int nMaxWeight)
{
	GPackage* pBestPackage=NULL;
	int nDistance=-1;
	
	GPackage* pPackage=(GPackage*) m_Packages.FirstObj();
	while(pPackage)
	{
		if(!pPackage->m_bPicked && pPackage->m_nOnBot==-1 && pPackage->m_nDistance!=0 && pPackage->m_nWeight<=nMaxWeight &&
			pPackage->m_nX==nSX && pPackage->m_nY==nSY && 
			(pPackage->m_nDistance<nDistance || !pBestPackage))
		{
			pBestPackage=pPackage;
			nDistance=pPackage->m_nDistance;
		}

		pPackage=(GPackage*) m_Packages.NextObj();		
	}
	return pBestPackage;
}


int GBoard::FindClosestBase(int nX, int nY, int nExclude)
{	
	int nBestBaseID=-1;
	int nDistance=-1;
	
	GPath* pPath;
	GHomeBase* pBase=(GHomeBase*) m_HomeBase.FirstObj();	
	while(pBase)
	{
		pPath=g_PathCache.FindPath(nX, nY, pBase->m_nX, pBase->m_nY);		
		if(pBase->m_nID!=nExclude && pBase->m_nPackageCnt!=0 && (pPath->GetDistance()<nDistance || nBestBaseID==-1) )
		{		
			nBestBaseID=pBase->m_nID;
			nDistance=pPath->GetDistance();
		}

		pBase=(GHomeBase*) m_HomeBase.NextObj();		
	}
	return nBestBaseID;
}


void GBoard::RemovePackage(int nPackage)
{
	GPackage* pPackage=(GPackage*) m_Packages.RemoveObj(nPackage);
	if(pPackage)
		delete pPackage;
}

GPackage* GBoard::FindPackage(int nPackage)
{
	return (GPackage*) m_Packages.FindObj(nPackage);
}

BOOL GBoard::ArePackagesHere(int nX, int nY)
{
	GPackage* pPackage=(GPackage*) m_Packages.FirstObj();
	while(pPackage)
	{
		if(pPackage->m_nDestX && pPackage->m_nDestY)
			return TRUE;
		pPackage=(GPackage*) m_Packages.NextObj();		
	}
	return FALSE;
}
