#include "GBoard.h"
#include "GPath.h"
#include "SStringList.h"
#include "main.h"
#include "stdio.h"


GPath::GPath(GBoard* pBoard)
{
	m_pBoard=pBoard;
//	m_asHeadNode=NULL;
	m_nDistance=0;
	m_nPath=NULL;
}
GPath::~GPath()
{
	if(m_nPath)
		delete[] m_nPath;
}


// ... Initialize board ...

// All spaces are valid, as long as they're within the board.
int GPath::Valid(int x, int y, void *p) 
{
	GBoard* pBoard=(GBoard*)p;
	if (x <= 0 || y <= 0 || x > pBoard->m_nWidth || y > pBoard->m_nHeight || 
		pBoard->m_pBoard[y-1][x-1]=='~' || pBoard->m_pBoard[y-1][x-1]=='#') 
			return FALSE;

	return TRUE;
}

// Cost is 1 per space.
int GPath::Cost(int x, int y, void *p) 
{
	return 1;
}

int GPath::FindPath(int nSX, int nSY, int nDX, int nDY)
{
	m_nSX=nSX;
	m_nSY=nSY;
	m_nDX=nDX;
	m_nDY=nDY;
	
	sprintf(m_sID, "%04i%04i%04i%04i", nSX, nSY, nDX, nDY);
	m_nDistance=0;
	CPathFinder	m_PathFinder;
	m_PathFinder.SetRows(m_pBoard->m_nHeight);
	m_PathFinder.SetData(m_pBoard);
	m_PathFinder.SetValid(Valid);
	m_PathFinder.SetCost(Cost);

	if (m_PathFinder.GeneratePath(nDX, nDY, nSX, nSY)) 
	{
		G_Printf("Path found...from %i,%i to %i,%i \n",nSX, nSY, nDX, nDY);
		
		_asNode* m_asHeadNode = m_PathFinder.GetBestNode();	

		_asNode* asCurNode=m_asHeadNode;
		while(asCurNode) 
		{
			m_nDistance++;
			//G_Printf("(%d, %d)->", asCurNode->x, asCurNode->y);
			asCurNode = asCurNode->parent;			
		}

		if(m_nDistance>0)
		{
			//copy path
			m_nPath=new GPoint[m_nDistance+1];
			asCurNode=m_asHeadNode;
			int i=0;
			while(asCurNode) 
			{
				//m_nPath[i]=new int(2);
				m_nPath[i].m_nX=asCurNode->x;
				m_nPath[i].m_nY=asCurNode->y;

				i++;						
				asCurNode = asCurNode->parent;			
			}
		}

		//G_Printf("\n");


	} 
	else 
		G_Printf("Cannot find a path... %i,%i to %i,%i \n",nSX, nSY, nDX, nDY);

	return m_nDistance;
}


void GPath::GetMove(int nMove, int* pnX, int *pnY)
{
	(*pnX)=m_nPath[nMove].m_nX;
	(*pnY)=m_nPath[nMove].m_nY;
}

GPathIterator::GPathIterator(GPath* pPath)
{
	m_pPath=pPath;	
	//m_asCurNode=m_pPath->GetHeadNode();
	//m_nPathArray=m_pPath->GetPath();
	m_nDistance=m_pPath->GetDistance();
	m_nCurMove=0;
}

void GPathIterator::Reset()
{
	//m_asCurNode=m_asHeadNode;
	m_nCurMove=0;
}

char GPathIterator::NextMove(int* pnX, int *pnY)
{
	int nLastX,nLastY,nNextX,nNextY;
	if(m_nCurMove>=m_nDistance)
		return 'X';
	
	m_pPath->GetMove(m_nCurMove, &nLastX, &nLastY);
	//nLastY=m_nPathArray[m_nCurMove][1];
	m_nCurMove++;	
	
	if(m_nCurMove>=m_nDistance)
		return 'X';

	m_pPath->GetMove(m_nCurMove, &nNextX, &nNextY);
	//nNextX=m_pPath->GetMove[m_nCurMove][0];//m_nPathArray[m_nCurMove][0];
	//nNextY=m_pPath->GetMove[m_nCurMove][1];//m_nPathArray[m_nCurMove][1];
	

	(*pnX)=nNextX;
	(*pnY)=nNextY;

	if(nNextX==nLastX+1)
		return 'E';
	if(nNextX==nLastX-1)
		return 'W';
	if(nNextY==nLastY+1)
		return 'N';
	if(nNextY==nLastY-1)
		return 'S';

	return 'X';
}
