
#include "SStringList.h"

#include <string.h>
#include <stdlib.h>


SStringList::SStringList()
{	
	InitVars();
}

SStringList::SStringList(const char* sString, char cSep, BOOL bTrimEnds, BOOL bQuouted)
{
	InitVars();
	Split(sString, cSep, bTrimEnds, bQuouted);
}

SStringList::~SStringList()
{
	if(m_sString)
		delete m_sString;
	
	if(m_npIndex)
		delete m_npIndex;

}

void SStringList::Split(const char* sString, char cSep, BOOL bTrimEnds, BOOL bQuouted)
{
	if(m_sString)
		delete m_sString;
	
	if(m_npIndex)
		delete m_npIndex;

	m_sString=new char[strlen(sString)+1];
	strcpy(m_sString, sString);

	IndexList(cSep, bTrimEnds, bQuouted);
}

void SStringList::InitVars()
{
	m_sString=NULL;
	m_npIndex=NULL;
	m_nItemCnt=0;
}

int SStringList::CharCount(char cChar)
{
	int nCnt=0;
	int nLen=strlen(m_sString);
	for(int i=0;i<nLen;i++)
		if(m_sString[i]==cChar)
			nCnt++;

	return nCnt;
}

int SStringList::IndexList(char cSep, BOOL bTrimEnds, BOOL bQuouted)
{
	m_nItemCnt=CharCount(cSep)+1;// this could get way too many if there is lots of quoted strings with the sep in them	

	if(m_npIndex)
		delete m_npIndex;

	m_npIndex=new int[m_nItemCnt];
	int nLen=strlen(m_sString);
	int nCurItem=0;

	// setup first item
	int i=0;
	BOOL bInQuoute=FALSE;
	if(m_sString[i]=='\'' || m_sString[i]=='\"')
	{
		bInQuoute=TRUE;
		i++;
	}
	m_npIndex[nCurItem]=i;
	nCurItem++;


	
	// find rest of items
	for(;i<nLen;i++)
	{
		if(m_sString[i]=='\'' || m_sString[i]=='\"')
		{
			if(bInQuoute)
			{
				bInQuoute=FALSE;
				m_sString[i]=' '; //set it to a space to chop off quotes
				i++;
			}
		}

		if(m_sString[i]==cSep && !bInQuoute)
		{	
			m_sString[i]=NULL; // end previous item

			if(m_sString[i+1]==cSep)
				m_npIndex[nCurItem]=-1;
			else				
			{
				i++;
				if(m_sString[i]=='\'' || m_sString[i]=='\"')
				{
					bInQuoute=TRUE;
					i++;
				}
				m_npIndex[nCurItem]=i;
				
			}			
			
			nCurItem++;
		}
	}

	//if(nCurItem!=m_nItemCnt)
	//	return 0;
	m_nItemCnt=nCurItem;


	if(bTrimEnds)
		TrimItems();

	return m_nItemCnt;
}

void SStringList::TrimItems()
{
	/*char* sItem;*/
	int i=0, n=0;
	for(int nItem=0;nItem<m_nItemCnt;nItem++)
	{
		i=m_npIndex[nItem];
		if(i==-1)
			continue;
		
		// trim leading
		while(m_sString[i] && m_sString[i]==' ')
			i++;

		m_npIndex[nItem]=i;
		
		// trim trailing
		n=(i+strlen(m_sString+m_npIndex[nItem]))-1;
		while(m_sString[n]==' ')
		{
			m_sString[n]=NULL;
			n--;
		}		
	}
}

const char* SStringList::GetItem(int nItem)
{
	if(nItem<0 || nItem>=m_nItemCnt || m_npIndex[nItem]==-1)
		return "";

	return m_sString+m_npIndex[nItem];
}

int SStringList::GetItemAsInt(int nItem)
{
	const char* sItem=GetItem(nItem);
	if(sItem)
		return atoi(sItem);
	else
		return 0;
}


const char* SStringList::operator[] (int nIndex)
{
	return GetItem(nIndex);
}


int SStringList::FindItem(const char* sItem)
{
	for(int i=0;i<m_nItemCnt;i++)
		if(strcmp(GetItem(i), sItem)==0)
			return i;

	return -1;
}