#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#ifdef WIN32
#include <conio.h>
#endif

#include "SSocket.h"
#include "SStringList.h"

#include "main.h"
#include "GBoard.h"
#include "GSmartBot.h"
#include "GObjList.h"
#include "GSmartBot.h"

BOOL UpdateBots();
BOOL UpdatePackages();

SSocket* pSocket;
int nID, nCapacity, nMoney;	
GBoard Board;
GObjList g_Bots;
GSmartBot* pOurBot;
GPathCache g_PathCache(&Board);

#define BUF_SIZE 1024*1024
char sBuf[BUF_SIZE];
unsigned int nSize;

int main(int argc, char** argv)
{
	int i;
	char sStr[1024];
	
	pSocket=new SSocket();
	if(argc==3)
	{
		printf("Connecting to: %s:%i\n",argv[1], atoi(argv[2]));
		pSocket->Connect(argv[1], atoi(argv[2]));
	}
	else
	{
		printf("Connecting to: icfp1.cse.ogi.edu:22101\n");
		pSocket->Connect("icfp1.cse.ogi.edu", 22101);
	}

	// send init
	strcpy(sStr, "Player\n");
	pSocket->Send(sStr, strlen(sStr));	

	//get board
	Board.ReceiveBoard(pSocket);
	

	// read starting params	
	nSize=BUF_SIZE;
	pSocket->RecvUntilStr(sBuf, &nSize, "\n", 1, FALSE,  0);
	sBuf[nSize]=NULL;
	SStringList sl(sBuf, ' ');
	nID=sl.GetItemAsInt(0);
	nCapacity=sl.GetItemAsInt(1);
	nMoney=sl.GetItemAsInt(2);	
	G_Printf("Params: ID=%i Capacity=%i Money=%i\n", nID, nCapacity, nMoney);

	Board.SetMaxWeight(nCapacity);
	// get initial bots
	UpdateBots();
	

	// main game loop
	BOOL bContinue=TRUE;	
	int nLoop=0;
	const char* sCmd;
	while(bContinue)
	{		
		// read list of packages
		if(!UpdatePackages())
		{
			G_Printf("Failed to read package updates\n");
			break;
		}

		//send command
		sCmd=pOurBot->GetNextCmd();		
		if(strchr(sCmd,'X'))
		{
			G_Printf("Invalid Cmd: %s", sCmd);
			break;
		}
		G_Printf("WeMoved: %s\n", sCmd);
		
		pSocket->SendLine(sCmd);

		// get bot updates
		if(!UpdateBots())
		{
			G_Printf("Failed to read bot updates\n");
			break;
		}

		pOurBot->OnAllBotsUpdated();

		nLoop++;
	}

	pSocket->Close();
	G_Printf("Dead. Press any key to exit.\n");
#ifdef WIN32
	getch();
#endif

	return 1;
}
/*
.....................
.........#.#.........
.........#.#.........
.........#.#.........
.........#.#.........
.........#.#.........
.........#.#.........
......####.####......
......#.......#......
#######.......#######
..........@..........
#######.......#######
......#.......#......
......####.####......
.........#.#.........
.........#.#.........
.........#.#.........
.........#.#.........
.........#.#.........
.........#.#.........

*/


BOOL UpdateBots()
{
	int i;
	nSize=BUF_SIZE;
	if(pSocket->RecvUntilStr(sBuf, &nSize, "\n", 1, FALSE,  0)!=TCP4U_SUCCESS)
		return FALSE;

	sBuf[nSize]=NULL;
	SStringList slBots(sBuf, '#');
	int nBot;
	GBot* pBot;

	// reset updated
	pBot=(GBot*)g_Bots.FirstObj();
	while(pBot)
	{
		pBot->SetUpdated(FALSE);
		pBot=(GBot*)g_Bots.NextObj();
	}

	// read updates
	for(i=1;i<slBots.Count();i++)
	{
		//#1 X 1 Y 1 E 		
		nBot=atoi(slBots[i]);
		pBot=(GBot*)g_Bots.FindObj(nBot);
		if(!pBot)
		{
			if(nBot==nID)
			{
				pOurBot=new GSmartBot(nID, nCapacity, nMoney, &Board);
				pBot=(GBot*)pOurBot;
			}
			else
				pBot=new GBot(nBot, nCapacity, nMoney, &Board);

			g_Bots.AddObj(nBot, (void*) pBot);
		}
		pBot->SetUpdated(TRUE);
		pBot->Update(slBots[i]);
	}

	// delete bots not updated
	pBot=(GBot*)g_Bots.FirstObj();
	while(pBot)
	{
		if(!pBot->GetUpdated())
		{
			g_Bots.RemoveObj(pBot->ID());
			pBot=(GBot*)g_Bots.FirstObj(); // start loop over again for lazy safety
		}
		else
			pBot=(GBot*)g_Bots.NextObj();
	}


	return TRUE;
}

BOOL UpdatePackages()
{
	int nPackage, nDestX, nDestY, nWeight;
	int i;
	nSize=BUF_SIZE;
	if(pSocket->RecvUntilStr(sBuf, &nSize, "\n", 1, FALSE,  0)!=TCP4U_SUCCESS)
		return FALSE;	
	
	if(nSize==0) // no packages in this spot
		return TRUE;

	sBuf[nSize]=NULL;

	// check for death reason
	if(sBuf[0]=='*')
	{
		G_Printf("%s\n", sBuf);
		return TRUE;
	}

	int nX=pOurBot->X();
	int nY=pOurBot->Y();	
	
	SStringList slPackage(sBuf, ' ');
	for(i=0;i<slPackage.Count();i+=4)
	{
		nPackage=slPackage.GetItemAsInt(i+0);
		nDestX=slPackage.GetItemAsInt(i+1);
		nDestY=slPackage.GetItemAsInt(i+2);
		nWeight=slPackage.GetItemAsInt(i+3);

		Board.AddPackage(nPackage, nX,nY, nDestX, nDestY, nWeight);
	}
	pOurBot->OnPackagesFound();

	return TRUE;
}


void G_Printf(const char* sFormat, ...)
{
#ifndef PRINT
	return;
#endif

	va_list ap;
	char sData[5000];

	va_start(ap, sFormat);
	vsprintf(sData,  sFormat, ap);
	va_end(ap);

    printf(sData);
}
