#include "GBot.h"
#include "SStringList.h"
#include "main.h"
#include "GPackage.h"
#include "GBoard.h"

GBot::GBot(int nID, int nMaxWeight, int nMoney, GBoard* pBoard)
{
	m_nID=nID;
	m_nMaxWeight=nMaxWeight;
	m_nMoney=nMoney;
	m_nCurWeight=0;
	m_pBoard=pBoard;
	m_nX=-1;
	m_nY=-1;
	m_nScore=0;
}

GBot::~GBot()
{
	// remove any packages this bot was carrying from play
	GPackage* pPackage=(GPackage*) m_Packages.FirstObj();
	if(pPackage)
	{
		m_pBoard->RemovePackage(pPackage->m_nID);
		pPackage=(GPackage*) m_Packages.NextObj();
	}
}

void GBot::Update(const char* sUpdates)
{	
	SStringList slCmds(sUpdates, ' ');	
	if(slCmds.Count()==1)
	{
		G_Printf("BotCmd: ID=%i no-op\n", m_nID);
		return;
	}

	char cCmd;
	int nValue;
	BOOL bMoved=FALSE;

	for(int nCmd=1;nCmd<slCmds.Count();nCmd++)
	{
		cCmd=(char)slCmds.GetItem(nCmd)[0];
		nValue=-1;
		
		switch(cCmd)
		{
		case 'X':
			nValue=slCmds.GetItemAsInt(++nCmd);
			m_nX=nValue;
			break;
		case 'Y':
			nValue=slCmds.GetItemAsInt(++nCmd);
			m_nY=nValue;
			break;
		case 'N':
			m_nY++;
			bMoved=TRUE;
			break;
		case 'S':
			m_nY--;
			bMoved=TRUE;
			break;
		case 'E':
			m_nX++;
			bMoved=TRUE;
			break;
		case 'W':
			m_nX--;
			bMoved=TRUE;
			break;
		case 'D':
			nValue=slCmds.GetItemAsInt(++nCmd);
			DroppedPackage(nValue);			
			break;
		case 'P':
			nValue=slCmds.GetItemAsInt(++nCmd);
			PickedupPackage(nValue);			
			break;
		}

		if(nValue==-1)
			G_Printf("BotCmd: ID=%i %c (%i,%i)\n", m_nID, cCmd, m_nX, m_nY);
		else
			G_Printf("BotCmd: ID=%i %c %i (%i,%i)\n", m_nID, cCmd, nValue, m_nX, m_nY);
	}
}

void GBot::PickedupPackage(int nPackage)
{
	GPackage* pPackage=m_pBoard->FindPackage(nPackage);
	if(pPackage)
	{
		m_nCurWeight+=pPackage->m_nWeight;
		pPackage->m_nOnBot=m_nID;
		m_Packages.AddObj(nPackage, pPackage);
	}
}

void GBot::DroppedPackage(int nPackage)
{
	m_Packages.RemoveObj(nPackage);

	GPackage* pPackage=m_pBoard->FindPackage(nPackage);
	if(pPackage)
	{
		m_nCurWeight-=pPackage->m_nWeight;

		// if it was dropped at it's destination then it's gone
		if(pPackage->m_nDestX==m_nX && pPackage->m_nDestY==m_nY)
		{
			m_nScore+=pPackage->m_nWeight;
			m_pBoard->RemovePackage(nPackage);
		}
	}
}

