#include "GSmartBot.h"
#include "GPath.h"
#include "GBoard.h"
#include "GPackage.h"
#include "stdio.h"

#include "main.h"

GSmartBot::GSmartBot(int nID, int nMaxWeight, int nMoney, GBoard* pBoard) : GBot(nID, nMaxWeight, nMoney, pBoard)
{
	m_pBoard=pBoard;	
	m_pPath=NULL;
	m_pPathIterator=NULL;

	m_nGoingToBase=-1;
	m_nDeliveringPackage=-1;
	//m_nPickUpPackage=-1;
	//m_pCurPackage=NULL;
	m_nDestX=-1;
	m_nDestY=-1;
	m_cCurCmd=NULL;
	m_nPushCount=0;
}

GSmartBot::~GSmartBot()
{
}

void GSmartBot::PickNextBase()
{
	m_nDeliveringPackage=-1;
	int nBase=m_pBoard->FindClosestBase(X(), Y(), m_nGoingToBase);	

	// may have been trying to find new base and didn't find one, so stick to the one we got
	if(nBase!=-1)
		m_nGoingToBase=nBase;

	if(m_nGoingToBase==-1)
		return;

	m_nDestX=m_pBoard->HomeBaseX(m_nGoingToBase);
	m_nDestY=m_pBoard->HomeBaseY(m_nGoingToBase);	
	if(m_nDestX==m_nX && m_nDestY==m_nY) // already there
	{
		//m_nGoingToBase=-1;
	}
	else
	{
		//m_pPath=g_PathCache.FindPath(X(), Y(), m_nDestX, m_nDestY);	
		m_pPath=g_PathCache.FindPath(X(), Y(), m_nDestX, m_nDestY);
		if(m_pPathIterator)
			delete m_pPathIterator;
		m_pPathIterator=new GPathIterator(m_pPath);
	}

	G_Printf("Going to Base: %i at %i,%i from %i,%i\n", m_nGoingToBase, m_nDestX, m_nDestY, X(), Y());
}

BOOL GSmartBot::PickNextPackageToDeliver()
{
	if(m_Packages.Count()>0)
	{
		GPackage* pPackage=(GPackage*)m_Packages.FirstObj();
		m_nDeliveringPackage=pPackage->m_nID;
		m_nDestX=pPackage->m_nDestX;
		m_nDestY=pPackage->m_nDestY;
		if(m_nDestX==m_nX && m_nDestY==m_nY) // already there
		{
			//m_nDeliveringPackage=-1;
		}
		else
		{
			m_pPath=g_PathCache.FindPath(m_nX, m_nY, m_nDestX, m_nDestY);			
			if(m_pPathIterator)
				delete m_pPathIterator;
			m_pPathIterator=new GPathIterator(m_pPath);
		}
		return TRUE;
	}
	else
	{
		m_nDeliveringPackage=-1;
		return FALSE;
	}	
}

const char* GSmartBot::GetNextCmd()
{
	int nBid=1;
	static char sCmd[16];
	BOOL bDone=FALSE;
	
	if(m_nDestX==m_nX && m_nDestY==m_nY && m_nDestX!=-1 && m_nDestY!=-1) //we're here
	{
		if(m_nDeliveringPackage!=-1) // drop packages if we were delivering		
		{
			int nPackagesDropped=0;
			if(m_Packages.Count()>1)
			{
				nBid=1+IsAdjacentBots()*28;
				sprintf(sCmd, "%i Drop ", nBid);				
				GPackage* pPackage=(GPackage*)m_Packages.FirstObj();
				while(pPackage)
				{
					if(pPackage->m_nDestX==m_nX && pPackage->m_nDestY==m_nY)
					{				
						sprintf(sCmd+strlen(sCmd), " %i",  pPackage->m_nID);
						nPackagesDropped++;
						bDone=TRUE;
					}
					pPackage=(GPackage*)m_Packages.NextObj();
				}
			}

			if(nPackagesDropped==0)
			{
				nBid=1+IsAdjacentBots()*2;
				if(!PickNextPackageToDeliver())
				{
					m_nDeliveringPackage=-1;
					PickNextBase();
				}
				m_cCurCmd=m_pPathIterator->NextMove(&m_nNextX, &m_nNextY);
				sprintf(sCmd, "%i Move %c", nBid, m_cCurCmd);
				bDone=TRUE;
			}

			//m_nDeliveringPackage=-1;				
			//m_pCurPackage=NULL;
		}
		else if(m_nGoingToBase!=-1) // we're at base, if code makes it here there were no packages
		{
			m_nGoingToBase=-1; // let the package check below get packages
			/*
			nBid=1+IsAdjacentBots()*5;
			G_Printf("No Packages at this base\n");
			PickNextBase();
			m_cCurCmd=m_pPathIterator->NextMove(&m_nNextX, &m_nNextY);
			sprintf(sCmd, "%i Move %c", nBid, m_cCurCmd);			
			bDone=TRUE;
			*/
		}
		else
		{
			PickNextBase();
			m_cCurCmd=m_pPathIterator->NextMove(&m_nNextX, &m_nNextY);
			sprintf(sCmd, "%i Move %c", nBid, m_cCurCmd);
			bDone=TRUE;
		}
	}
	
	if(!bDone && m_pBoard->ArePackagesHere(m_nX, m_nY)) // check for packages
	{
		m_pBoard->ResetPickedPackages();

		int nCurWeight=m_nCurWeight;
		nBid=1+IsAdjacentBots()*28;
		sprintf(sCmd, "%i Pick", nBid);
		while(nCurWeight<m_nMaxWeight)
		{
			GPackage* pPackage=m_pBoard->FindBestDelivery(m_nX, m_nY, m_nMaxWeight-nCurWeight);
			if(pPackage)
			{
				nCurWeight+=pPackage->m_nWeight;
				pPackage->m_bPicked=TRUE;
				sprintf(sCmd+strlen(sCmd), " %i",  pPackage->m_nID);				
				bDone=TRUE;
			}				
			else
				break;
		}
	}
	
	if(!bDone)// didn't do anything else go to next spot
	{
		if(m_nGoingToBase==-1 && m_nDeliveringPackage==-1)
		{
			PickNextBase();
		}

		m_cCurCmd=m_pPathIterator->NextMove(&m_nNextX, &m_nNextY);
		nBid=1+IsAdjacentBots()*2;
		sprintf(sCmd, "%i Move %c", nBid, m_cCurCmd);		
	}

	if(m_cCurCmd=='X')   // we're confused, do a no-op and hope we recover
	{			
		nBid=1;
		sprintf(sCmd, "%i Pick %i", nBid, 12345);
	}
	m_nMoney-=nBid;

	return sCmd;
}


void GSmartBot::OnAllBotsUpdated()
{
	// see if we moved onto our target base, if so reset package count before packages come in
	if(m_nGoingToBase!=-1)
	{
		GHomeBase* pBase=m_pBoard->IsHomeBase(m_nX, m_nY);
		if(pBase)
			m_pBoard->ResetBasePackageCount(m_nX, m_nY);
	}

	// check to see if we got pushed
	if(m_nNextX!=m_nX || m_nNextY!=m_nY)
	{
		m_nPushCount++;
		G_Printf("We were pushed: %i!\n", m_nPushCount);
		g_PathCache.ReleasePath(m_pPath);

		// if we got pushed five times in a row going to a base, lets try a different base
		if(m_nPushCount>5 && m_nGoingToBase!=-1)
			PickNextBase();
		else
		{
			m_pPath=g_PathCache.FindPath(m_nX, m_nY, m_nDestX, m_nDestY);	
			if(m_pPathIterator)
				delete m_pPathIterator;
			m_pPathIterator=new GPathIterator(m_pPath);
		}
	}
	else
		m_nPushCount=0;
}

void GSmartBot::OnPackagesFound()
{
}

void GSmartBot::PickedupPackage(int nPackage)
{
	GBot::PickedupPackage(nPackage);

	if(m_nDeliveringPackage==-1)
	{
		PickNextPackageToDeliver();
	}

	//if(m_nPickUpPackage==nPackage)
	{
	//	m_nPickUpPackage=-1;
	//	m_nDeliveringPackage=nPackage;
		
		/*
		m_pPath=m_pCurPackage->m_pPath;
		if(m_pPathIterator)
			delete m_pPathIterator;
		m_pPathIterator=new GPathIterator(m_pPath);
		*/
	}
}

void GSmartBot::DroppedPackage(int nPackage)
{
	GBot::DroppedPackage(nPackage);

	// all packages delivered or dropped from being pushed, next GetNextCmd will find new base
	if(m_Packages.Count()==0)
		m_nDeliveringPackage=-1; 
}

BOOL GSmartBot::IsAdjacentBots()
{
	// reset updated
	GBot* pBot=(GBot*)g_Bots.FirstObj();
	while(pBot)
	{
		if(pBot->X()==m_nX-1 ||pBot->X()==m_nX+1 ||
				pBot->Y()==m_nY-1 ||pBot->Y()==m_nY+1)
			return TRUE;

		pBot=(GBot*)g_Bots.NextObj();
	}
	return FALSE;
}