//////////////////////////////////////////////////////////////////
// CPathFinder class (2/7/2000)
// Author: James Matthews
//
// Implements the A* algorithm.
// Written by Generation5 (http://www.generation5.org/)
//
// See http://www.generation5.org/astar.shtml for details.

#ifndef _PATHFINDER_H_
#define _PATHFINDER_H_

#include <memory.h>
#include "SDefs.h"

class _asNode {
	public:
		_asNode(int a = -1,int b = -1) : x(a), y(b), number(0), numchildren(0) {
			parent = next = NULL;
			memset(children, 0, sizeof(children));
		}

		int			f,g,h;			// Bad name, but that's the terminology.
		int			x,y;			// Coordinate position
		int			numchildren;
		int			number;			// x*30+y
		_asNode		*parent;
		_asNode		*next;
		_asNode		*children[8];	// Assumes square tiles
};

// Stack for propagation.
struct _asStack {
	_asNode	 *data;
	_asStack *next;
};

typedef int(*_asFunc)(int, int, void *);

class CPathFinder {
	public:
		CPathFinder();
		~CPathFinder();

		bool	GeneratePath(int, int, int, int);
		void	SetValid(_asFunc sv) { udValid = sv;   }
		void	SetCost(_asFunc sc)  { udCost = sc;    }
		void	SetData(void *sd)	 { m_pCBData = sd; }
		void	SetRows(int r)		 { m_iRows = r;    }

		_asNode	*GetBestNode() { return m_pBest; }

	protected:
		int		m_iRows;			// Used to calculate node->number
		int		m_iSX, m_iSY, m_iDX, m_iDY, m_iDNum;
		void	*m_pCBData;			// Data passed back.

		_asNode	*m_pOpen;			// The open list
		_asNode	*m_pClosed;			// The closed list
		_asNode *m_pBest;			// The best node
		_asStack*m_pStack;			// Propagation stack

		_asFunc	udCost;				// The user-defined cost function
		_asFunc udValid;			// The user-defined validity function

		// Functions.
		void	AddToOpen(_asNode *);
		void	ClearNodes();
		void	CreateChildren(_asNode *);
		void	LinkChild(_asNode *, int, int);
		void	UpdateParents(_asNode *);

		// Stack Functions.
		void	Push(_asNode *);
		_asNode *Pop();
		
		_asNode *CheckList(_asNode *, int);
		_asNode	*GetBest();
		
		// Inline functions.
		inline int Coord2Num(int x, int y) { return x * m_iRows + y; }
};

#endif
