


#include <stdio.h>

#include <sys/types.h>
#include "string.h"
#include <errno.h>


#ifdef __UNIX__
#include <sys/types.h>       
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>	
#include <unistd.h>
#include <fcntl.h>

#include "SSocket.h"
#include "tcp4u.h"

#else // windows

#include "tcp4u.h"
#include "SSocket.h"
#endif

#ifdef __UNIX__
	#include <sys/time.h>
	#include <sys/types.h>
	#include <unistd.h> 
#endif


SSocket::SSocket(SOCKET& Socket)
{
	m_Socket=Socket;
	m_bLocalSocket=FALSE;
	Tcp4uInit();
}

SSocket::SSocket(BOOL bLocalSocket)
{
	m_bLocalSocket=bLocalSocket;
	
	if(!m_bLocalSocket)
		Tcp4uInit();

	m_Socket=-1;
}
	
SSocket::~SSocket()
{	
	/*
	#ifdef __UNIX__
	fcntl(m_Socket, F_SETFL, O_NONBLOCK);	
	#endif
	*/
	//Close();	
}

SOCKET SSocket::GetSOCKET()
{
	return m_Socket;
}

int SSocket::Abort()
{
	return TcpAbort();
}

/*
int SSocket::TcpAccept (SOCKET *pCSock, SOCKET ListenSock, UINT nTO)
{
	return TcpAccept (*pCSock, ListenSock, nTO);
}
*/


int SSocket::Connect (LPCSTR szServer, unsigned short lpPort)
{		
	if(m_bLocalSocket)
		return TCP4U_SUCCESS;
	else
		return TcpConnect (&m_Socket, szServer,  NULL, &lpPort);
}

void SSocket::ShutDown()
{
	// shutdown is winsock 2, may not be supported on Win95

}

int SSocket::Close ()
{
	if(m_bLocalSocket)
		return TCP4U_SUCCESS;
	else
	{
#ifdef __UNIX__		
		fcntl(m_Socket, F_SETFL, O_NONBLOCK);
		struct linger sling = { FALSE, 0 };	 //TRUE
		setsockopt (m_Socket, SOL_SOCKET, SO_LINGER,(LPSTR) &sling, sizeof sling);
		shutdown(m_Socket, 2);
		int nRet=close(m_Socket);
		m_Socket=-1;
		return nRet;
#else
		int nRet=TcpClose (&m_Socket);
		m_Socket=-1;
		return nRet;
#endif
	}
		
}

int SSocket::Flush()
{
	return TcpFlush(m_Socket);
}



int SSocket::Accept(SOCKET* pSocket, int nTO)
{
	//*pSocket=new SSocket;
	
//	int nRet=TcpAccept(pSocket, m_Socket, nTO);

	
#ifdef __UNIX__
	struct sockaddr_in saSockAddr;
	unsigned int    nAddrLen = sizeof saSockAddr;	
#else
	sockaddr saSockAddr;
	 int    nAddrLen = sizeof saSockAddr;	 
#endif

	SOCKET             ConSock;
	ConSock = accept (m_Socket, (struct sockaddr far *) &saSockAddr, &nAddrLen);

	if (ConSock==INVALID_SOCKET)   
		return TCP4U_ERROR;
	else
	{
		*pSocket = ConSock;
		return TCP4U_SUCCESS;
	}
		


	//if(nRet!=TCP4U_SUCCESS)
	//	delete *pSocket;

	//return nRet;
}





/*
int SSocket::TcpGetListenSocket (SOCKET *pS, LPCSTR szService,
                          unsigned short *lpPort, int nPendingConnection)
{
}
*/

int SSocket::Recv (LPSTR szBuf, unsigned uBufSize, unsigned uTimeOut)
{
	//return TcpRecv (m_Socket,  szBuf,  uBufSize,  uTimeOut,  HFILE_ERROR);
	int             Rc, nUpRc;  /* Return Code of select and recv */
	struct timeval  TO;         /* Time Out structure             */
	struct timeval *pTO;        /* Time Out structure             */
	fd_set          ReadMask;   /* select mask                    */

	if (m_Socket==INVALID_SOCKET)
		return TCP4U_ERROR;

	FD_ZERO (& ReadMask);     
	FD_SET (m_Socket, & ReadMask); 
	
	switch (uTimeOut)
	{
	case  TCP4U_WAITFOREVER:
		pTO = NULL; 
		break;
	case  TCP4U_DONTWAIT:
		TO.tv_sec = TO.tv_usec=0 ; 
		pTO = & TO;
	break;	
	default :
		TO.tv_sec = (long) uTimeOut;
		TO.tv_usec=0;
		pTO = & TO;
	break;
	}	

	Rc = select (m_Socket+1, & ReadMask, NULL, NULL, pTO);
	if(Rc<0)
		return  TCP4U_ERROR;
	if(Rc==0)
		return  TCP4U_TIMEOUT; 
	if(szBuf==NULL  ||  uBufSize==0)  
		return TCP4U_SUCCESS;
	
	Rc = recv (m_Socket, szBuf, uBufSize, 0);	
	if(Rc==SOCKET_ERROR)
		nUpRc = TCP4U_ERROR;
	else if(Rc==0)
		nUpRc = TCP4U_SOCKETCLOSED; 
	else
		nUpRc=Rc;	


	return nUpRc;
}

int SSocket::Send ( LPCSTR szBuf, unsigned uBufSize, BOOL bHighPriority)
{
	//return TcpSend (m_Socket,  szBuf,  uBufSize,  bHighPriority,  HFILE_ERROR);
	int nSent;

	if (m_Socket==INVALID_SOCKET)
		return TCP4U_ERROR;

	nSent=send(m_Socket, szBuf, uBufSize, 0);
	

	if(nSent==-1)
	{	

		return -1;
	}
	return nSent;
}

BOOL SSocket::IsDataAvail ()
{
	return TcpIsDataAvail (m_Socket);
}

BOOL SSocket::IsOOBDataAvail()
{
	return TcpIsOOBDataAvail(m_Socket);
}

/* PP protocole (2 first bytes contain length of data) */
int SSocket::PPRecv (LPSTR szBuf, unsigned uBufSize, unsigned uTimeOut, BOOL bExact)
{
	return TcpPPRecv (m_Socket,  szBuf,  uBufSize,  uTimeOut, 
                      bExact,  HFILE_ERROR);
}

int SSocket::PPSend ( LPCSTR szBuf, unsigned uBufSize)
{
	return TcpPPSend (m_Socket,  szBuf,  uBufSize,  HFILE_ERROR);
}

/* Recv Until family */
int SSocket::RecvUntilStr (LPSTR szBuf,unsigned *lpBufSize,
                          LPSTR szStop, unsigned uStopSize, BOOL bCaseSensitive,
                          unsigned uTimeOut)
{
	return TcpRecvUntilStr (m_Socket,  szBuf, lpBufSize,
                           szStop,  uStopSize,  bCaseSensitive,
                           uTimeOut,  HFILE_ERROR);
}


int SSocket::ReadMultiLine ( LPSTR szBuf, UINT BufSize, UINT uTimeOut)
{
	return TnReadMultiLine (m_Socket,  szBuf,  BufSize,  uTimeOut, HFILE_ERROR);
}

int SSocket::ReadLine (LPSTR szBuf, UINT BufSize, UINT uTimeOut)
{
	return TnReadLine (m_Socket ,  szBuf,  BufSize,  uTimeOut, HFILE_ERROR);
}
		  /*
int SSocket::SendMultiLine ( LPCSTR szString, BOOL bEnd)
{
	return TnSendMultiLine (m_Socket ,  szString,  bEnd, NULL);
}			*/

int SSocket::SendLine ( LPCSTR szString, BOOL bHighPriority)
{
	return TnSend (m_Socket ,  szString,  bHighPriority, HFILE_ERROR);
}

int SSocket::GetAnswerCode(LPSTR szInBuf, UINT uBufSize, UINT uTimeOut)
{
	return TnGetAnswerCode(m_Socket , szInBuf, uBufSize, uTimeOut, HFILE_ERROR);
}


int SSocket::SetNonBlocking()
{
#ifdef __UNIX
   int opts;

   opts = fcntl(m_Socket,F_GETFL);
   if (opts < 0) 
		return opts;

   opts = (opts | O_NONBLOCK);
	return fcntl(m_Socket,F_SETFL,opts);
#endif
	  /*
#ifdef __WIN32__
	u_long  nonblock = TRUE;
	ioctlsocket(m_Socket, FIONBIO, (u_long FAR*)&nonblock);
	return TCP4U_SUCCESS;
#endif */
	return -1;
}







 /*
typedef BOOL (CALLBACK far *TRANSFER_CBK) (
#ifdef NEED_PROTO
    long, long, long, LPSTR
#endif
);
int SSocket::TcpRecvUntilClosedEx (SOCKET       *pCSock,     LPCSTR szLocalFile, 
                                TRANSFER_CBK  CbkTransmit,
                                unsigned      uTimeout,   unsigned int  uBufSize, 
                                long          lUserValue, long          lTotalBytes);

*/